var DmarcianStatusIcon = function (icon) {
    this.icon = icon;
};

DmarcianStatusIcon.templates = {};

DmarcianStatusIcon.prototype = {
    constructor: DmarcianStatusIcon,

    render: function (state) {
        this.icon.html((DmarcianStatusIcon.templates[state])
            ? DmarcianStatusIcon.templates[state]
            : ''
        );
    }
};

var DmarcianInput = function (input) {
    this.input = input;
};

DmarcianInput.prototype = {
    constructor: DmarcianInput,

    val: function () {
        return this.input.val().trim();
    },

    error: function () {
        this.input.css('border-color', 'red');
    },

    reset: function () {
        this.input.css('border-color', '');
        this.input.val('');
    }
};

var DmarcianStatusPanel = function () {};

DmarcianStatusPanel.templates = {};

DmarcianStatusPanel.prototype = {
    constructor: DmarcianStatusPanel,

    events: {
        stateChanged: 'dmarcian:state-changed'
    },

    init: function ($, panel, api) {
        this.$ = $;
        this.panel = panel;
        this.api = api;

        // select available slots
        this.errorSlot = panel.find('.error-slot');
    },

    render: function () {
        // update all global DKIM icons
        this.updateGlobalDKIMIcons();

        // bind all events
        this.bindDMARCEvents();
        this.bindSPFEvents();
        this.bindDKIMEvents();

        this.panel.css('display', '');
    },

    stateChanged: function () {
        this.$(document).trigger(this.events.stateChanged);
    },

    bindDMARCEvents: function () {
        var self = this;

        this.panel.on('click', '.dmarc .refresh', function () {
            var button = self.$(this);
            var elements = self.getDomainElements(button);

            elements.icon.render('loading');

            self.api.checkDMARC(elements.domain, function (response, status) {
                elements.icon.render(response.state);
                self.stateChanged();
                self.showResponseError(response, status);
            });
        });
    },

    bindSPFEvents: function () {
        var self = this;

        this.panel.on('click', '.spf .refresh', function () {
            var button = self.$(this);
            var elements = self.getDomainElements(button);

            elements.icon.render('loading');

            self.api.checkSPF(elements.domain, function (response, status) {
                elements.icon.render(response.state);
                self.stateChanged();
                self.showResponseError(response, status);
            });
        });
    },

    bindDKIMEvents: function () {
        var self = this;

        this.panel.on('click', '.dkim .add-selector', function (event) {
            event.preventDefault();

            var button = self.$(this);
            var elements = self.getDomainElements(button);
            var selector = elements.input.val();

            // skip empty values
            // prevent duplicate selectors
            if (!selector || self.isDKIMSelectorPresent(selector, elements.selectorsSlot)) {
                elements.input.error();
                return;
            }

            // reset input state
            elements.input.reset();

            // add new selector detail row
            var selectorDetail = DmarcianStatusPanel.templates.selectorDetail.replace(/SELECTOR/g, selector);
            elements.selectorsSlot.append(selectorDetail);
            elements.selectorsSlot.find('.refresh').last().click();
            elements.selectorsRow.css('display', 'table-row');
        });

        this.panel.on('click', '.selector-detail .remove-selector', function () {
            var button = self.$(this);
            var elements = self.getSelectorElements(button);

            // update table
            elements.detail.remove();
            if (elements.row.find('.selector-detail').length == 0) {
                elements.row.css('display', 'none');
            }

            // update backend db
            self.api.deleteDKIM(elements.domain, elements.selector, function () {
                self.stateChanged();
            });

            // update global icon
            self.updateGlobalDKIMIcon(
                elements.globalIcon,
                elements.row
            );
        });

        this.panel.on('click', '.selector-detail .refresh', function () {
            var button = self.$(this);
            var elements = self.getSelectorElements(button);

            elements.icon.render('loading');
            elements.globalIcon.render('loading');

            self.api.checkDKIM(elements.domain, elements.selector, function (response, status) {
                elements.icon.render(response.state);
                self.stateChanged();
                self.showResponseError(response, status);

                // update global icon
                self.updateGlobalDKIMIcon(
                    elements.globalIcon,
                    elements.row
                );
            });
        });
    },

    isDKIMSelectorPresent: function (selector, selectorsSlot) {
        var self = this;
        var present = false;

        // go through all existing selectors
        var selectors = selectorsSlot.find('.selector-detail .selector');
        selectors.each(function (index, existing) {
            var compareSelector = self.$(existing).html().trim();
            if (selector == compareSelector) {
                present = true;
            }
        });

        return present;
    },

    updateGlobalDKIMIcon: function (icon, row) {
        // create a map between simple numbers and states
        var states = ['unknown', 'invalid', 'valid', 'valid'];

        // determine the valid and problem bit values
        var validBit = Boolean(row.find('.selector-detail .status-icon .state-valid').length) | 0;
        var problemBit = Boolean(row.find(
            '.selector-detail .status-icon .state-invalid,' +
            '.selector-detail .status-icon .state-error'
        ).length) | 0;

        // render state based on the bit map
        var index = parseInt(validBit.toString() + problemBit.toString(), 2);
        icon.render(states[index]);
    },

    updateGlobalDKIMIcons: function () {
        var self = this;

        this.panel.find('.selector-detail .refresh').each(function () {
            var button = self.$(this);
            var elements = self.getSelectorElements(button);

            // update global icon
            self.updateGlobalDKIMIcon(
                elements.globalIcon,
                elements.row
            );
        });
    },

    getDomainElements: function (button) {
        var domainPolicy = button.parents('.domain-policy');
        var domainRow = domainPolicy.parents('.domain-row');
        var selectorsRow = domainRow.next('.selectors-row');

        return {
            domain: domainRow.find('.domain').html(),
            input: new DmarcianInput(domainPolicy.find('.input')),
            icon: new DmarcianStatusIcon(domainPolicy.find('.status-icon')),
            selectorsRow: selectorsRow,
            selectorsSlot: selectorsRow.find('.selectors-slot')
        };
    },

    getSelectorElements: function (button) {
        var detail = button.parents('.selector-detail');
        var row = detail.parents('.selectors-row');
        var domainRow = row.prev('.domain-row');

        return {
            domain: domainRow.find('.domain').html(),
            selector: detail.find('.selector').html(),
            row: row,
            detail: detail,
            icon: new DmarcianStatusIcon(detail.find('.status-icon')),
            globalIcon: new DmarcianStatusIcon(domainRow.find('.dkim .status-icon'))
        }
    },

    showResponseError: function (response, status) {
        if (status === 200 && response.hasOwnProperty('error')) {
            this.showError('Oops! Something unexpected happened. Send us an email via ' +
                '<a href="mailto:wp-support@dmarcian.com?subject=WP%20dmarcian%20plugin%20error&amp;body=' +
                'Error%20code%20(do%20NOT%20modify)%20' + response.error + '" target="_blank">this</a> link.');
        }
        if (status !== 200) {
            this.showError('Oops! Something unexpected happened.');
        }
    },

    showError: function (message) {
        this.errorSlot.html(DmarcianStatusPanel.templates.error.replace(
            'MESSAGE', message
        ));
    }
};

var DmarcianProgressPanel = function () {};

DmarcianProgressPanel.prototype = {
    constructor: DmarcianProgressPanel,

    init: function (panel) {
        this.panel = panel;
        this.progress = panel.find('.progress');
    },

    render: function () {
        this.panel.css('display', '');
    },

    updateProgress: function (progress) {
        this.progress.css('width', progress + '%');
    }
};

var dmarcianProgressPanel = new DmarcianProgressPanel();
var dmarcianStatusPanel = new DmarcianStatusPanel();

jQuery(document).ready(function () {
    // define a utility function for loading templates
    var findTemplate = function (context, id) {
        var template = context.find('#' + id);
        return (template.length) ? template.html().trim() : '';
    };

    // set icon templates
    var statusPanel = jQuery('.status-panel');
    DmarcianStatusIcon.templates.unknown = findTemplate(statusPanel, 'template-state-unknown');
    DmarcianStatusIcon.templates.loading = findTemplate(statusPanel, 'template-state-loading');
    DmarcianStatusIcon.templates.valid = findTemplate(statusPanel, 'template-state-valid');
    DmarcianStatusIcon.templates.invalid = findTemplate(statusPanel, 'template-state-invalid');
    DmarcianStatusIcon.templates.error = findTemplate(statusPanel, 'template-state-error');

    // set panel templates
    DmarcianStatusPanel.templates.error = findTemplate(statusPanel, 'template-error');
    DmarcianStatusPanel.templates.selectorDetail = findTemplate(statusPanel, 'template-selector-detail');

    // kick it
    dmarcianProgressPanel.init(jQuery('.progress-panel'));
    dmarcianStatusPanel.init(jQuery, statusPanel, dmarcianAPI);
});