var DmarcianMenu = function (logo, refreshProgress) {
    this.logo = logo;
    this.refreshProgress = refreshProgress;
    this.updateLogo();
};

DmarcianMenu.prototype = {
    constructor: DmarcianMenu,

    renderGreenLogo: function () {
        this.resetLogo();
        this.logo.addClass('green-logo');
    },

    renderRedLogo: function () {
        this.resetLogo();
        this.logo.addClass('red-logo');
    },

    resetLogo: function () {
        this.logo.removeClass('green-logo red-logo');
    },

    updateRefreshProgress: function (progress) {
        this.refreshProgress.css('display', '');
        this.refreshProgress.html(' (' + progress + '%)');
    },

    updateLogo: function () {
        var self = this;

        dmarcianAPI.getGlobalState(function (data) {
            if (data.valid) {
                self.renderGreenLogo();
            } else {
                self.renderRedLogo();
            }
        });
    }
};

jQuery(document).ready(function () {
    var isDashboard = (typeof dmarcianProgressPanel !== 'undefined');
    var menu = new DmarcianMenu(
        jQuery('#adminmenu #toplevel_page_dmarcian .wp-menu-image'),
        jQuery('.dmarcian-refresh-progress')
    );

    // do nothing if we have a multisite installation
    if (dmarcianParams.multisite) {
        return;
    }

    // kick of the refresh process when ordered to
    if (dmarcianParams.refreshCache) {
        dmarcianAPI.refreshCache();
    }

    // show refresh progress if there is one available
    if (dmarcianParams.refreshCache || dmarcianParams.refreshInProgress) {
        // do we have a progress panel to show?
        if (isDashboard) {
            dmarcianProgressPanel.render();
        }

        // check progress regularly
        var interval = setInterval(function () {
            dmarcianAPI.refreshCacheProgress(function (data) {
                if (data.progress === false) {
                    clearInterval(interval);
                    location.reload();
                    return;
                }

                menu.updateRefreshProgress(data.progress);
                if (isDashboard) {
                    dmarcianProgressPanel.updateProgress(data.progress);
                }
            });
        }, 1000);

        return;
    }

    // render the dashboard if there is nothing to refresh
    if (isDashboard) {
        dmarcianStatusPanel.render();

        // bind state changed handler
        jQuery(document).on(dmarcianStatusPanel.events.stateChanged, function () {
            menu.updateLogo();
        });
    }
});