<?php

/**
 * Renders the HTML for a valid state icon.
 *
 * @return string
 */
function dmarcian_render_state_valid()
{
    return '<span class="dashicons dashicons-yes state-valid" style="color:green"></span>';
}

/**
 * Renders the HTML for an invalid state icon.
 *
 * @return string
 */
function dmarcian_render_state_invalid()
{
    return '<span class="dashicons dashicons-no state-invalid" style="color:red"></span>';
}

/**
 * Renders the HTML for an error state icon.
 *
 * @return string
 */
function dmarcian_render_state_error()
{
    return '<span class="dashicons dashicons-editor-help state-error" style="color:red"></span>';
}

/**
 * Renders the HTML for a unknown state icon.
 *
 * @return string
 */
function dmarcian_render_state_unknown()
{
    return '<span class="dashicons dashicons-editor-help state-unknown"></span>';
}

/**
 * Renders the HTML for the state icon with a variable state.
 *
 * @param string $state
 * @return string
 */
function dmarcian_render_state( $state )
{
    switch ( $state ) {
        case Dmarcian_Cache::STATE_VALID:
            return dmarcian_render_state_valid();
        case Dmarcian_Cache::STATE_INVALID:
            return dmarcian_render_state_invalid();
        case Dmarcian_Cache::STATE_ERROR:
            return dmarcian_render_state_error();
        default:
            return dmarcian_render_state_unknown();
    };
}

/**
 * Renders the HTML for a selector detail table row.
 *
 * @param string $selector
 * @param string $state
 * @return string
 */
function dmarcian_render_selector_detail( $selector, $state )
{
    $selectorSpan = '<span class="selector">' . $selector . '</span>';
    $statusIcon = '<span class="status-icon">' . dmarcian_render_state( $state ) . '</span>';
    $refreshButton = '<button class="refresh">refresh</button>';
    $removeButton = '<button class="remove-selector">remove</button>';

    return sprintf(
        '<tr class="selector-detail"><td>%s</td><td>%s%s</td><td>%s</td>',
        $selectorSpan,
        $statusIcon,
        $refreshButton,
        $removeButton
    );
}

?>

<div class="wrap dmarcian-inspector">
    <h1>About</h1>

    <p>
        dmarcian plugin provides on-demand and daily checks of the email authentication status for your domain
        directly from your admin panel. The plugin will automatically recognize the domain and inspect in
        real-time the SPF, DKIM* and DMARC records assigned to your domain. The verdict would be the technical
        accuracy of your records. While each authentication technology may support different policies
        (e.g. DMARC p=none vs. p=reject), the plugin would not take into account the current policy and report
        only on the record validity.
    </p>

    <p>(*) DKIM: you'd need to supply your DKIM selector (s=) in order to enable the DKIM check.</p>

    <p>
        Learn more about the protocols:<br/>
        DMARC: article about <a href="https://dmarc.org/" target="_blank">dmarc.org</a> and
        <a href="https://space.dmarcian.com/video-dmarc-overview/" target="_blank">video</a> overview.<br/>
        SPF: <a href="https://en.wikipedia.org/wiki/Sender_Policy_Framework" target="_blank">wiki</a> page and
        <a href="https://space.dmarcian.com/video-spf-overview/" target="_blank">video</a> overview.<br/>
        DKIM: <a href="https://en.wikipedia.org/wiki/DomainKeys_Identified_Mail" target="_blank">wiki</a> page and
        <a href="https://space.dmarcian.com/video-dkim-overview/" target="_blank">video</a> overview.<br/>
    </p>

    <p>
        Ready to start? See how to add a DMARC record in monitoring mode (no impact)
        <a href="https://space.dmarcian.com/how-to-publish-a-dmarc-record/" target="_blank">here</a>.
    </p>

    <p>
        Not sending email but still want to protect your domains from abuse -- you sure can. Learn how
        <a href="https://dmarc.org/wiki/FAQ#I_have_domains_that_do_not_send_emails.2C_how_can_I_protect_them.3F"
           target="_blank">here</a>.
    </p>

    <h1>Dashboard</h1>
    <p></p>

    <?php if ( $multisite ) { ?>
        <div class="notice notice-error">
            <p>Sorry but we do NOT support multisite yet.</p>
        </div>
    <?php } ?>

    <div class="progress-panel" style="display:none">
        <p>Please wait a few seconds while we check your status.</p>

        <div class="progress-wrap" style="width:200px;height:20px;background-color:white">
            <div class="progress" style="width:0;height:100%;background-color:green"></div>
        </div>
    </div>

    <div class="status-panel" style="display:none">
        <div class="error-slot"></div>

        <table class="widefat striped domain-table">
            <thead>
                <tr>
                    <th>Domain</th>
                    <th>DMARC</th>
                    <th>SPF</th>
                    <th>DKIM</th>
                </tr>
            </thead>
            <tbody>
                <?php foreach ($domains as $domain => $states) { ?>
                    <tr class="domain-row">
                        <td class="domain"><?php echo $domain; ?></td>
                        <td class="dmarc domain-policy">
                            <span class="status-icon">
                                <?php echo dmarcian_render_state( $states['dmarc'] ); ?>
                            </span>
                            <button class="refresh">refresh</button>
                        </td>
                        <td class="spf domain-policy">
                            <span class="status-icon">
                                <?php echo dmarcian_render_state( $states['spf'] ); ?>
                            </span>
                            <button class="refresh">refresh</button>
                        </td>
                        <td class="dkim domain-policy">
                            <span class="status-icon">
                                <?php echo dmarcian_render_state_unknown(); ?>
                            </span>
                            <form style="display:inline">
                                <input type="text" size="10" class="input" />
                                <button class="add-selector" type="submit">add</button>
                            </form>
                        </td>
                    </tr>
                    <?php $selectorsDisplay = ( count( $states['dkim'] ) ) ? '' : 'display:none'; ?>
                    <tr class="selectors-row" style="<?php echo $selectorsDisplay; ?>">
                        <td></td>
                        <td colspan="3">
                            <table class="widefat">
                                <tr>
                                    <th>Selector</th>
                                    <th>Status</th>
                                    <th>Actions</th>
                                </tr>
                                <tbody class="selectors-slot">
                                    <?php foreach ( $states['dkim'] as $selector => $state ) { ?>
                                        <?php echo dmarcian_render_selector_detail( $selector, $state ); ?>
                                    <?php } ?>
                                </tbody>
                            </table>
                        </td>
                    </tr>
                <?php } ?>
            </tbody>
        </table>

        <script type="text/template" id="template-error">
            <div class="notice notice-error"><p>MESSAGE</p></div>
        </script>

        <script type="text/template" id="template-state-loading">
            <span class="dashicons" style="background:url('/wp-admin/images/loading.gif') no-repeat center"></span>
        </script>

        <script type="text/template" id="template-state-valid">
            <?php echo dmarcian_render_state_valid(); ?>
        </script>

        <script type="text/template" id="template-state-invalid">
            <?php echo dmarcian_render_state_invalid(); ?>
        </script>

        <script type="text/template" id="template-state-unknown">
            <?php echo dmarcian_render_state_unknown(); ?>
        </script>

        <script type="text/template" id="template-state-error">
            <?php echo dmarcian_render_state_error(); ?>
        </script>

        <script type="text/template" id="template-selector-detail">
            <?php echo dmarcian_render_selector_detail( 'SELECTOR', '' ); ?>
        </script>
    </div>

    <p>Interested in checking a domain different than the above? Try these:</p>

    <p>
        <a href="https://dmarcian.com/dmarc-inspector/?utm_source=wp&utm_campaign=dmarc" target="_blank">DMARC Inspector</a> |
        <a href="https://dmarcian.com/spf-survey/?utm_source=wp&utm_campaign=spf" target="_blank">SPF Surveyor</a> |
        <a href="https://dmarcian.com/dkim-inspector/?utm_source=wp&utm_campaign=dkim" target="_blank">DKIM Inspector</a>
    </p>
</div>