<?php

/**
 * Class Dmarcian_State is a pure data object representing the result of an API call.
 */
class Dmarcian_State
{
    /**
     * @var string Check state supported by the Dmarcian_Cache.
     */
    protected $state;

    /**
     * @var array The data returned by the Dmarcian API.
     */
    protected $data = array();

    /**
     * @var array A mixed error storage.
     */
    protected $errors = array();

    /**
     * The state is always valid by default.
     */
    public function __construct()
    {
        $this->state = Dmarcian_Cache::STATE_VALID;
    }

    /**
     * @return string
     */
    public function get_state()
    {
        return $this->state;
    }

    /**
     * @return bool
     */
    public function is_error()
    {
        return $this->state == Dmarcian_Cache::STATE_ERROR;
    }

    /**
     * Sets the state to invalid.
     */
    public function invalid()
    {
        $this->state = Dmarcian_Cache::STATE_INVALID;
    }

    /**
     * Sets the state to error - there was some communication problem.
     *
     * Provides fluent interface.
     *
     * @param array $errors Default: array().
     * @return Dmarcian_State
     */
    public function error( $errors = array() )
    {
        $this->state = Dmarcian_Cache::STATE_ERROR;
        $this->errors = $errors;

        return $this;
    }

    /**
     * @return array
     */
    public function get_data()
    {
        return $this->data;
    }

    /**
     * @param array $data Default: array().
     */
    public function set_data( $data = array() )
    {
        $this->data = $data;
    }

    /**
     * Returns the errors as base64 encoded string so users can send them to us.
     *
     * @return string
     */
    public function get_error_string()
    {
        return base64_encode( serialize( $this->errors ) );
    }
}