<?php

/**
 * Class Dmarcian_Actions helps implementing AJAX actions.
 */
class Dmarcian_Actions extends Dmarcian_Singleton
{
    /**
     * @const NONCE_ACTION Key for nonce generation and verification.
     */
    const NONCE_ACTION = 'dmarcian';

    /**
     * @const NONCE_KEY POST parameter to check for the presence of a nonce.
     */
    const NONCE_KEY = 'security';

    /**
     * @var string Required capability for calling the API.
     */
    protected $capability;

    /**
     * Returns certain status code as a response.
     *
     * This way we can handle different types of problems consistently.
     *
     * @param int $code
     */
    protected function return_status( $code )
    {
        wp_die( '', '', $args = array( 'response' => $code ) );
    }

    /**
     * Verifies that a request has access.
     */
    protected function check_access()
    {
        // verify nonce
        check_ajax_referer( static::NONCE_ACTION, static::NONCE_KEY );

        // a capability MUST be present
        if ( ! $this->capability || ! current_user_can( $this->capability ) ) {
            $this->return_status( 403 );
        }
    }

    /**
     * Dmarcian_Actions constructor. Initializes all base shared configuration.
     *
     * @param array $params
     */
    protected function __construct( $params = array() )
    {
        if ( array_key_exists( 'capability', $params ) ) {
            $this->capability = $params['capability'];
        }
    }
}