<?php

/**
 * Class Dmarcian_Options provides unified access to all plugin options.
 */
class Dmarcian_Config
{
    /**
     * @const DB_VERSION Name of the option that hold the current database version.
     */
    const DB_VERSION = 'dmarcian_db_version';

    /**
     * @return bool|string
     */
    public static function get_db_version()
    {
        return get_option( self::DB_VERSION );
    }

    /**
     * @param string $version
     */
    public static function set_db_version( $version )
    {
        update_option( self::DB_VERSION, $version );
    }

    /**
     * @const CACHE_REFRESH_OPTION Last time the cache was refreshed.
     */
    const CACHE_REFRESH = 'dmarcian_cache_refresh';

    /**
     * @const CACHE_REFRESH_FORMAT Format of the cache refresh date.
     */
    const CACHE_REFRESH_FORMAT = 'Y-m-d';

    /**
     * @return bool
     */
    public static function is_cache_old()
    {
        return ( get_option( self::CACHE_REFRESH ) != date( self::CACHE_REFRESH_FORMAT ) );
    }

    /**
     * Marks that the cache has been refreshed today.
     */
    public static function set_cache_is_fresh()
    {
        update_option( self::CACHE_REFRESH, date( self::CACHE_REFRESH_FORMAT ) );
    }

    /**
     * @const CACHE_REFRESH_PROGRESS Stores the progress of a cache refresh.
     */
    const CACHE_REFRESH_PROGRESS = 'dmarcian_cache_refresh_progress';

    /**
     * @return bool
     */
    public static function is_cache_refresh_in_progress()
    {
        return ( false !== self::get_cache_refresh_progress() );
    }

    /**
     * @return int|false
     */
    public static function get_cache_refresh_progress()
    {
        return get_option( self::CACHE_REFRESH_PROGRESS );
    }

    /**
     * @param int $processed
     * @param int $total
     */
    public static function set_cache_refresh_progress( $processed, $total )
    {
        update_option( self::CACHE_REFRESH_PROGRESS, (int) ( ($processed / $total) * 100 ) );
    }

    /**
     * Lets us check whether we have a progress in place or not.
     */
    public static function delete_cache_refresh_progress()
    {
        delete_option( self::CACHE_REFRESH_PROGRESS );
    }

    /**
     * Removes all set options.
     */
    public static function clean()
    {
        delete_option( self::DB_VERSION );
        delete_option( self::CACHE_REFRESH );
        delete_option( self::CACHE_REFRESH_PROGRESS );
    }
}