<?php

/**
 * Class Dmarcian_Migrations manages the current database version and all necessary
 * actions when upgrading to a newer version.
 */
class Dmarcian_Migrations
{
    /**
     * @const DB_VERSION Latest database version.
     */
    const DB_VERSION = '1.0';

    /**
     * Updates the database to the latest version available.
     */
    public static function update()
    {
        // check if there is anything to upgrade
        if ( Dmarcian_Config::get_db_version() ) {
            return;
        }

        // we do not have a version yet so create the table
        global $wpdb;
        $table_name = Dmarcian_Cache::get_table_name();
        $charset_collate = $wpdb->get_charset_collate();

        $sql = "CREATE TABLE IF NOT EXISTS $table_name (
		        id VARCHAR(100) NOT NULL,
		        domain VARCHAR(255) NOT NULL,
		        selector VARCHAR(255),
		        policy VARCHAR(5) NOT NULL,
                state VARCHAR(10) NOT NULL,
		        PRIMARY KEY  (id)
	    ) $charset_collate;";

        require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
        dbDelta( $sql );

        Dmarcian_Config::set_db_version( self::DB_VERSION );
    }

    /**
     * Removes all artifacts from the database.
     */
    public static function clean()
    {
        global $wpdb;
        $table_name = Dmarcian_Cache::get_table_name();
        $wpdb->query( "DROP TABLE IF EXISTS $table_name;" );
    }
}